/*
 * (c) Copyright 2015 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#ifndef DCAMPROF_H_
#define DCAMPROF_H_

#include <stdio.h>
#include <stdbool.h>

#include <colmath.h>

struct cam2xyz_patch {
    spectrum_t *spectrum;
    char str_id[8];
    bool emissive;
    int class;
    double k_lch[3];
    double m_de;
    double de;
    double w;
    v3 cam;
    v3 xyz;
    v3 tmp;
};

enum tc_type {
    TC_NEUTRAL,
    TC_STANDARD
};

enum lookop_type {
    LOOKOP_STRETCH,
    LOOKOP_ADDHUE,
    LOOKOP_ADDCHROMA,
    LOOKOP_ADDLIGHTNESS,
    LOOKOP_SCALECHROMA,
    LOOKOP_SCALELIGHTNESS,
    LOOKOP_SETTEMPERATURE,
    LOOKOP_CURVES
};

enum lookop_vs_x {
    LOOKOP_VS_X_LIGHTNESS,
    LOOKOP_VS_X_CHROMA,
    LOOKOP_VS_X_HUE,
    LOOKOP_VS_X_HSL_LIGHTNESS,
    LOOKOP_VS_X_HSL_SATURATION,
    LOOKOP_VS_X_HSL_HUE,
    LOOKOP_VS_X_HSV_VALUE,
    LOOKOP_VS_X_HSV_SATURATION,
    LOOKOP_VS_X_HSV_HUE
};

enum lookop_curve_type {
    LOOKOP_CURVE_SPLINE,
    LOOKOP_CURVE_ROUNDEDSTEP,
    LOOKOP_CURVE_LINEAR
};

struct lookop_curve {
    enum lookop_curve_type type;
    int handle_count;
    double handles[];
};

struct lookop_vs {
    enum lookop_vs_x x_type;
    double xrange[2];
    double yscale;
    struct lookop_curve *curve;
};

struct lookop {
    enum lookop_type type;
    union {
        struct {
            double value;
        } gen;
        struct {
            int dim_count;
            enum lookop_vs_x dims[3];
            double ranges[3][2];
            struct lookop_curve *curves[3];
        } stretch;
        struct {
            bool is_srgb_gamma;
            double gamma;
            struct lookop_curve *curves[3];
        } curves;
        struct {
            double xy[2];
        } settemp;
    } p;
    bool blend_invert;
    bool blend_rgb;
    int vs_count;
    struct lookop_vs *vs;
};

struct tone_rep_op_config {
    double chroma_scaling;
    struct {
        double adjust_factor;
        double low_chroma;
        double high_chroma;
    } saturated;
    struct {
        double adjust_factor;
        double low_lightness;
        double high_lightness;
    } shadows;
    struct {
        struct lookop_curve *keep_factor;
        struct lookop_curve *low_satscale;
        struct lookop_curve *high_satscale;
    } rolloff;
    int lookop_count;
    struct lookop *lookops;
};

struct patch_set {
    size_t patch_count;
#define TARGET_MAX_CLASS_COUNT 24
#define TARGET_CLASS_NAME_SIZE 24
    char class_names[TARGET_MAX_CLASS_COUNT][TARGET_CLASS_NAME_SIZE];
    struct cam2xyz_patch patch[];
};

struct dcam_ssf {
    char *camera_name;
    struct observer ssf;
};

struct chromalut_t_;
typedef struct chromalut_t_ chromalut_t;

struct dcam_profile {
    enum exif_lightsource illuminant_name;
    spectrum_t *illuminant_spec;
    v3 illuminant_wp;
    m3x3 color_matrix;
    m3x3 forward_matrix;
    v3 forward_matrix_wb;
    char *camera_name;
    chromalut_t *lut;
};

void
dcam_profile_delete(struct dcam_profile *prof);

FILE *
open_log_file(const char dir[],
              const char name[]);

struct patch_set *
normalize_patch_set(const struct patch_set *ps);

#endif
